<script setup lang="ts">
import { computed, ref } from 'vue';

import { useVbenDrawer } from '@vben/common-ui';
import { $t } from '@vben/locales';
import { addFullName, cloneDeep, getPopupContainer, listToTree } from '@vben/utils';

import { useVbenForm } from '#/adapter/form';
import { addFixtureType, getFixtureType, listFixtureType, updateFixtureType } from '#/api/eims/fixture-type';

import { drawerSchema } from './data';

interface ModalProps {
  id?: number | string;
  update: boolean;
}

const emit = defineEmits<{ reload: [] }>();

const isUpdate = ref(false);
const title = computed(() => {
  return isUpdate.value ? $t('pages.common.edit') : $t('pages.common.add');
});

const [BasicForm, formApi] = useVbenForm({
  commonConfig: {
    componentProps: {
      class: 'w-full'
    },
    formItemClass: 'col-span-2',
    labelWidth: 90
  },
  schema: drawerSchema(),
  showDefaultActions: false,
  wrapperClass: 'grid-cols-2'
});

async function setupFixtureTypeSelect() {
  // menu
  const menuArray = await listFixtureType({ status: '0' });
  // support i18n
  menuArray.forEach((item) => {
    item.typeName = $t(item.typeName);
  });
  // const folderArray = menuArray.filter((item) => item.menuType === 'M');
  const menuTree = listToTree(menuArray, { id: 'id', pid: 'parentId' });
  const fullMenuTree = [
    {
      id: 0,
      typeName: $t('menu.root'),
      children: menuTree
    }
  ];
  addFullName(fullMenuTree, 'typeName', ' / ');
  formApi.updateSchema([
    {
      componentProps: {
        fieldNames: {
          label: 'typeName',
          value: 'id'
        },
        getPopupContainer,
        // 设置弹窗滚动高度 默认256
        listHeight: 300,
        showSearch: true,
        treeData: fullMenuTree,
        treeDefaultExpandAll: false,
        // 默认展开的树节点
        treeDefaultExpandedKeys: [0],
        treeLine: { showLeafIcon: false },
        // 筛选的字段
        treeNodeFilterProp: 'typeName',
        treeNodeLabelProp: 'fullName'
      },
      fieldName: 'parentId'
    }
  ]);
}

const [BasicDrawer, drawerApi] = useVbenDrawer({
  onCancel: handleCancel,
  onConfirm: handleConfirm,
  async onOpenChange(isOpen) {
    if (!isOpen) {
      return null;
    }
    drawerApi.drawerLoading(true);
    const { id, update } = drawerApi.getData() as ModalProps;
    isUpdate.value = update;

    // 加载设备类型树选择
    await setupFixtureTypeSelect();
    if (id) {
      await formApi.setFieldValue('parentId', id);
      if (update) {
        const record = await getFixtureType(id);
        await formApi.setValues(record);
      }
    }
    drawerApi.drawerLoading(false);
  }
});

async function handleConfirm() {
  try {
    drawerApi.drawerLoading(true);
    const { valid } = await formApi.validate();
    if (!valid) {
      return;
    }
    const data = cloneDeep(await formApi.getValues());
    await (isUpdate.value ? updateFixtureType(data) : addFixtureType(data));
    emit('reload');
    await handleCancel();
  } catch (error) {
    console.error(error);
  } finally {
    drawerApi.drawerLoading(false);
  }
}

async function handleCancel() {
  drawerApi.close();
  await formApi.resetForm();
}
</script>

<template>
  <BasicDrawer :close-on-click-modal="false" :title="title" class="w-[600px]">
    <BasicForm />
  </BasicDrawer>
</template>
